" Vim indent file
" Language:	Scheme
" Maintainer:	Sergey Khorev <sergey.khorev@gmail.com>
" Last Change:	2005 Jun 24

" Only load this indent file when no other was loaded.
if exists("b:did_indent")
  finish
endif

runtime! indent/lisp.vim
