" A series of tests that can run in one Vim invocation.
" This makes testing go faster, since Vim doesn't need to restart.

source test_assign.vim
source test_bufline.vim
source test_cd.vim
source test_changedtick.vim
source test_cursor_func.vim
source test_delete.vim
source test_ex_undo.vim
source test_ex_z.vim
source test_execute_func.vim
source test_expand.vim
source test_expand_dllpath.vim
source test_expr.vim
source test_feedkeys.vim
source test_file_perm.vim
source test_fileformat.vim
source test_filetype.vim
source test_filter_cmd.vim
source test_filter_map.vim
source test_findfile.vim
source test_float_func.vim
source test_fnamemodify.vim
source test_functions.vim
source test_ga.vim
source test_glob2regpat.vim
source test_global.vim
source test_goto.vim
source test_help_tagjump.vim
source test_join.vim
source test_jumps.vim
source test_lambda.vim
source test_lispwords.vim
source test_mapping.vim
source test_match.vim
source test_menu.vim
source test_messages.vim
source test_partial.vim
source test_popup.vim
source test_put.vim
source test_recover.vim
source test_reltime.vim
source test_scroll_opt.vim
source test_searchpos.vim
source test_set.vim
source test_sort.vim
source test_sha256.vim
source test_statusline.vim
source test_syn_attr.vim
source test_tabline.vim
source test_tabpage.vim
source test_tagcase.vim
source test_tagjump.vim
source test_taglist.vim
source test_timers.vim
source test_true_false.vim
source test_unlet.vim
source test_virtualedit.vim
source test_window_cmd.vim
