" Vim syntax file
" Language:		Innovation Data Processing upstream.rpt file
" Maintainer:		Rob Owens <rowens@fdrinnovation.com>
" Latest Revision:	2014-03-13

" Quit when a syntax file was already loaded
if exists("b:current_syntax")
  finish
endif

setlocal foldmethod=syntax

" Parameters:
syn keyword upstreamdat_Parameter ACCEPTPCREMOTE 
syn keyword upstreamdat_Parameter ACCEPTREMOTE 
syn keyword upstreamdat_Parameter ACTION 
syn keyword upstreamdat_Parameter ACTIVATEONENTRY 
syn keyword upstreamdat_Parameter ARCHIVEBIT 
syn keyword upstreamdat_Parameter ARCHIVEBIT 
syn keyword upstreamdat_Parameter ASCTOEBC 
syn keyword upstreamdat_Parameter ASRBACKUP 
syn keyword upstreamdat_Parameter ATTENDED 
syn keyword upstreamdat_Parameter AUTHORITATIVE 
syn keyword upstreamdat_Parameter AUTHORITATIVERESTORE 
syn keyword upstreamdat_Parameter AUTHORITATIVERESTORE 
syn keyword upstreamdat_Parameter BACKUPPROFILE 
syn keyword upstreamdat_Parameter BACKUPPROFILE2 
syn keyword upstreamdat_Parameter BACKUPREPARSEFILES 
syn keyword upstreamdat_Parameter BACKUPREPARSEFILES 
syn keyword upstreamdat_Parameter BACKUPVERIFY 
syn keyword upstreamdat_Parameter BLANKTRUNC 
syn keyword upstreamdat_Parameter CALCDASDSIZE 
syn keyword upstreamdat_Parameter CHANGEDIRATTRIBS 
syn keyword upstreamdat_Parameter CHANGEDIRATTRIBS 
syn keyword upstreamdat_Parameter COMPRESSLEVEL 
syn keyword upstreamdat_Parameter CONTROLFILE 
syn keyword upstreamdat_Parameter DASDOVERRIDE 
syn keyword upstreamdat_Parameter DATELIMIT 
syn keyword upstreamdat_Parameter DATELIMIT 
syn keyword upstreamdat_Parameter DAYSOLD 
syn keyword upstreamdat_Parameter DAYSOLD 
syn keyword upstreamdat_Parameter DELETED 
syn keyword upstreamdat_Parameter DELETED 
syn keyword upstreamdat_Parameter DELETEPROMPTS 
syn keyword upstreamdat_Parameter DELETEPROMPTS 
syn keyword upstreamdat_Parameter DESTINATION 
syn keyword upstreamdat_Parameter DESTINATION 
syn keyword upstreamdat_Parameter DIRDELETE 
syn keyword upstreamdat_Parameter DIRECTORVMC 
syn keyword upstreamdat_Parameter DIRONLYRESTOREOK 
syn keyword upstreamdat_Parameter DIRSONLY 
syn keyword upstreamdat_Parameter DIRSONLY 
syn keyword upstreamdat_Parameter DISASTERRECOVERY 
syn keyword upstreamdat_Parameter DISPLAY 
syn keyword upstreamdat_Parameter DRIVEALIAS 
syn keyword upstreamdat_Parameter DRIVEALIAS 
syn keyword upstreamdat_Parameter DUALCOPY 
syn keyword upstreamdat_Parameter DUPDAYS 
syn keyword upstreamdat_Parameter DUPLICATE 
syn keyword upstreamdat_Parameter EBCTOASC 
syn keyword upstreamdat_Parameter ENCRYPT 
syn keyword upstreamdat_Parameter ENCRYPTLEVEL 
syn keyword upstreamdat_Parameter EXCLUDELISTNAME 
syn keyword upstreamdat_Parameter FAILBACKUPONERROR 
syn keyword upstreamdat_Parameter FAILBACKUPONERROR 
syn keyword upstreamdat_Parameter FAILIFNOFILES 
syn keyword upstreamdat_Parameter FAILIFNOFILES 
syn keyword upstreamdat_Parameter FAILIFSKIP 
syn keyword upstreamdat_Parameter FAILJOB 
syn keyword upstreamdat_Parameter FAILRESTOREONERROR 
syn keyword upstreamdat_Parameter FAILRESTOREONERROR 
syn keyword upstreamdat_Parameter FILEDATE 
syn keyword upstreamdat_Parameter FILEDATE 
syn keyword upstreamdat_Parameter FILEDELETE 
syn keyword upstreamdat_Parameter FILEDELETE 
syn keyword upstreamdat_Parameter FILES 
syn keyword upstreamdat_Parameter FILES 
syn keyword upstreamdat_Parameter FILESOPENFORUPDAT 
syn keyword upstreamdat_Parameter FILESOPENFORUPDAT 
syn keyword upstreamdat_Parameter FILETRANSFER 
syn keyword upstreamdat_Parameter GETREMOTEFILES 
syn keyword upstreamdat_Parameter HARDLINKDB 
syn keyword upstreamdat_Parameter HARDLINKS 
syn keyword upstreamdat_Parameter HARDLINKS 
syn keyword upstreamdat_Parameter HIDDENFILES 
syn keyword upstreamdat_Parameter HIDDENFILES 
syn keyword upstreamdat_Parameter HOLDTAPE 
syn keyword upstreamdat_Parameter HOLDUSERDIRS 
syn keyword upstreamdat_Parameter HOSTFILENAME 
syn keyword upstreamdat_Parameter HOSTRECORD 
syn keyword upstreamdat_Parameter HOSTSORT 
syn keyword upstreamdat_Parameter IGNOREPLUGINSFORRESTORE 
syn keyword upstreamdat_Parameter INCRDB 
syn keyword upstreamdat_Parameter INCRDBARCHIVEBIT 
syn keyword upstreamdat_Parameter INCRDBDELETEDFILES 
syn keyword upstreamdat_Parameter INCREMENTAL 
syn keyword upstreamdat_Parameter INCREMENTAL 
syn keyword upstreamdat_Parameter INQOPTIONS 
syn keyword upstreamdat_Parameter INSTALLWIN2KAGENT 
syn keyword upstreamdat_Parameter INSTALLWIN2KAGENT 
syn keyword upstreamdat_Parameter JOBOPTIONS 
syn keyword upstreamdat_Parameter JOBRETURNCODEMAP 
syn keyword upstreamdat_Parameter JOBWAITTIMELIMIT 
syn keyword upstreamdat_Parameter KEEPALIVE 
syn keyword upstreamdat_Parameter LANINTERFACE 
syn keyword upstreamdat_Parameter LANWSNAME 
syn keyword upstreamdat_Parameter LANWSPASSWORD 
syn keyword upstreamdat_Parameter LASTACCESS 
syn keyword upstreamdat_Parameter LASTACCESS 
syn keyword upstreamdat_Parameter LATESTDATE 
syn keyword upstreamdat_Parameter LATESTDATE 
syn keyword upstreamdat_Parameter LATESTTIME 
syn keyword upstreamdat_Parameter LATESTTIME 
syn keyword upstreamdat_Parameter LATESTVERSION 
syn keyword upstreamdat_Parameter LINEBLOCK 
syn keyword upstreamdat_Parameter LINETRUNC 
syn keyword upstreamdat_Parameter LISTENFORREMOTE 
syn keyword upstreamdat_Parameter LOCALBACKUP 
syn keyword upstreamdat_Parameter LOCALBACKUPDIR 
syn keyword upstreamdat_Parameter LOCALBACKUPMAX 
syn keyword upstreamdat_Parameter LOCALBACKUPMAXFILESIZE 
syn keyword upstreamdat_Parameter LOCALBACKUPMAXSIZE 
syn keyword upstreamdat_Parameter LOCALEXCLUDEFILE 
syn keyword upstreamdat_Parameter LOCALPARAMETERS 
syn keyword upstreamdat_Parameter LOCALPASSWORD 
syn keyword upstreamdat_Parameter LOCALRESTORE 
syn keyword upstreamdat_Parameter LOCALUSER 
syn keyword upstreamdat_Parameter LOFS 
syn keyword upstreamdat_Parameter LOGNONFATAL 
syn keyword upstreamdat_Parameter MAXBACKUPFILESFAIL 
syn keyword upstreamdat_Parameter MAXBACKUPTIME 
syn keyword upstreamdat_Parameter MAXDUPS 
syn keyword upstreamdat_Parameter MAXFILENAMESIZE 
syn keyword upstreamdat_Parameter MAXKFILESIZE 
syn keyword upstreamdat_Parameter MAXLOGDAYS 
syn keyword upstreamdat_Parameter MAXRESTOREFILESFAIL 
syn keyword upstreamdat_Parameter MAXRESTORETIME 
syn keyword upstreamdat_Parameter MAXRETRY 
syn keyword upstreamdat_Parameter MAXRPTDAYS 
syn keyword upstreamdat_Parameter MERGE 
syn keyword upstreamdat_Parameter MIGRBITS 
syn keyword upstreamdat_Parameter MIGRBITS 
syn keyword upstreamdat_Parameter MINCOMPRESSSIZE 
syn keyword upstreamdat_Parameter MINIMIZE 
syn keyword upstreamdat_Parameter MODIFYFILE 
syn keyword upstreamdat_Parameter MOUNTPOINTS 
syn keyword upstreamdat_Parameter MOUNTPOINTS 
syn keyword upstreamdat_Parameter NDS 
syn keyword upstreamdat_Parameter NDS 
syn keyword upstreamdat_Parameter NEWFILECOMPARE 
syn keyword upstreamdat_Parameter NFSBELOW 
syn keyword upstreamdat_Parameter NODATAOK 
syn keyword upstreamdat_Parameter NODIRFORINCREMENTAL 
syn keyword upstreamdat_Parameter NODIRFORINCREMENTAL 
syn keyword upstreamdat_Parameter NONFILEDATABITMAP 
syn keyword upstreamdat_Parameter NONFILEDATABITMAP 
syn keyword upstreamdat_Parameter NOPOINTRESTORE 
syn keyword upstreamdat_Parameter NOSPECINHERITANCE 
syn keyword upstreamdat_Parameter NOTIFYEVENTS 
syn keyword upstreamdat_Parameter NOTIFYFAILUREATTACHMENT 
syn keyword upstreamdat_Parameter NOTIFYSUCCESSATTACHMENT 
syn keyword upstreamdat_Parameter NOTIFYTARGETS 
syn keyword upstreamdat_Parameter NOUIDGIDNAMES 
syn keyword upstreamdat_Parameter NOUIDGIDNAMES 
syn keyword upstreamdat_Parameter NOVELLMIGRATE 
syn keyword upstreamdat_Parameter NOVELLMIGRATE 
syn keyword upstreamdat_Parameter NOVELLMIGRATEADDEXT 
syn keyword upstreamdat_Parameter NOVELLMIGRATEADDEXT 
syn keyword upstreamdat_Parameter NOVELLPROFILE 
syn keyword upstreamdat_Parameter NOVELLRECALL 
syn keyword upstreamdat_Parameter NTFSADDPERMISSION 
syn keyword upstreamdat_Parameter NTFSADDPERMISSION 
syn keyword upstreamdat_Parameter NTREGRESTORE 
syn keyword upstreamdat_Parameter OSTYPE 
syn keyword upstreamdat_Parameter OUTPORT 
syn keyword upstreamdat_Parameter PACKFLUSHAFTERFILE 
syn keyword upstreamdat_Parameter PACKRECSIZE 
syn keyword upstreamdat_Parameter PARAMETER 
syn keyword upstreamdat_Parameter PASSWORD 
syn keyword upstreamdat_Parameter PATHNAME 
syn keyword upstreamdat_Parameter PATHNAME 
syn keyword upstreamdat_Parameter PERFORMBITMAP 
syn keyword upstreamdat_Parameter PERFORMNUMRECORDS 
syn keyword upstreamdat_Parameter PERFORMRECORDSIZE 
syn keyword upstreamdat_Parameter PLUGIN 
syn keyword upstreamdat_Parameter PLUGIN 
syn keyword upstreamdat_Parameter PLUGINPARAMETERS 
syn keyword upstreamdat_Parameter PLUGINPARAMETERS 
syn keyword upstreamdat_Parameter POSTJOB 
syn keyword upstreamdat_Parameter PREJOB 
syn keyword upstreamdat_Parameter PRTYCLASS 
syn keyword upstreamdat_Parameter PRTYLEVEL 
syn keyword upstreamdat_Parameter RECALLCLEANUP 
syn keyword upstreamdat_Parameter RECALLOFFLINEFILES 
syn keyword upstreamdat_Parameter RECALLOFFLINEFILES 
syn keyword upstreamdat_Parameter RECORDSIZE 
syn keyword upstreamdat_Parameter REMOTEADDR 
syn keyword upstreamdat_Parameter REMOTEAPPLPREF 
syn keyword upstreamdat_Parameter REMOTEAPPLRETRY 
syn keyword upstreamdat_Parameter REMOTECONNECTTYPE 
syn keyword upstreamdat_Parameter REMOTEFLAGS 
syn keyword upstreamdat_Parameter REMOTEIPADAPTER 
syn keyword upstreamdat_Parameter REMOTELOCALPARAMETERS 
syn keyword upstreamdat_Parameter REMOTELOGMODE 
syn keyword upstreamdat_Parameter REMOTELUNAME 
syn keyword upstreamdat_Parameter REMOTEMAXRETRIES 
syn keyword upstreamdat_Parameter REMOTEMODENAME 
syn keyword upstreamdat_Parameter REMOTEPARAMETERFILE 
syn keyword upstreamdat_Parameter REMOTEPORT 
syn keyword upstreamdat_Parameter REMOTEREQUEST 
syn keyword upstreamdat_Parameter REMOTERESTART 
syn keyword upstreamdat_Parameter REMOTEROUTE 
syn keyword upstreamdat_Parameter REMOTETARGETNAME 
syn keyword upstreamdat_Parameter REMOTETCP 
syn keyword upstreamdat_Parameter REMOTETIMEOUT 
syn keyword upstreamdat_Parameter REMOTETMAXRETRY 
syn keyword upstreamdat_Parameter REMOTETPN 
syn keyword upstreamdat_Parameter REMOTEUSAPPL 
syn keyword upstreamdat_Parameter REMOTEVERIFY 
syn keyword upstreamdat_Parameter REMOTEWTOCOMP 
syn keyword upstreamdat_Parameter REPORTNAME 
syn keyword upstreamdat_Parameter REPORTOPTIONS 
syn keyword upstreamdat_Parameter RESTARTLASTFILE 
syn keyword upstreamdat_Parameter RESTART 
syn keyword upstreamdat_Parameter RESTARTTYPE 
syn keyword upstreamdat_Parameter RESTARTVERSIONDATE 
syn keyword upstreamdat_Parameter RESTOREARCHIVEBIT 
syn keyword upstreamdat_Parameter RESTORECHECKPOINT 
syn keyword upstreamdat_Parameter RESTOREDATELIMIT 
syn keyword upstreamdat_Parameter RESTOREDATELIMIT 
syn keyword upstreamdat_Parameter RESTOREFILEFAIL 
syn keyword upstreamdat_Parameter RESTOREMOUNTPOINTS 
syn keyword upstreamdat_Parameter RESTOREMOUNTPOINTS 
syn keyword upstreamdat_Parameter RESTORESEGMENTS 
syn keyword upstreamdat_Parameter RESTORESEGMENTS 
syn keyword upstreamdat_Parameter RESTORETODIFFFS 
syn keyword upstreamdat_Parameter RETAIN 
syn keyword upstreamdat_Parameter RETAIN 
syn keyword upstreamdat_Parameter ROOTENTRY 
syn keyword upstreamdat_Parameter ROOTENTRY 
syn keyword upstreamdat_Parameter SAN 
syn keyword upstreamdat_Parameter SCHEDULENAME 
syn keyword upstreamdat_Parameter SEGMENTEDFILESIZE 
syn keyword upstreamdat_Parameter SEGMENTEDFILESIZE 
syn keyword upstreamdat_Parameter SEGMENTSIZE 
syn keyword upstreamdat_Parameter SEGMENTSIZE 
syn keyword upstreamdat_Parameter SENDHOSTDETAILS 
syn keyword upstreamdat_Parameter SINGLEFS 
syn keyword upstreamdat_Parameter SIZETRC 
syn keyword upstreamdat_Parameter SKIP 
syn keyword upstreamdat_Parameter SKIPBACKUPSCAN 
syn keyword upstreamdat_Parameter SKIPOLD 
syn keyword upstreamdat_Parameter SKIPOLD 
syn keyword upstreamdat_Parameter SMSTARGETSERVICENAME 
syn keyword upstreamdat_Parameter SMSTSA 
syn keyword upstreamdat_Parameter SOLO 
syn keyword upstreamdat_Parameter SORTBACKUP 
syn keyword upstreamdat_Parameter SOSDISK 
syn keyword upstreamdat_Parameter SOSDISK 
syn keyword upstreamdat_Parameter SOSTIMESTAMP 
syn keyword upstreamdat_Parameter SOSTIMESTAMP 
syn keyword upstreamdat_Parameter SOSTIMESTAMPPATH 
syn keyword upstreamdat_Parameter SOSTIMESTAMPPATH 
syn keyword upstreamdat_Parameter SPECNUMBER 
syn keyword upstreamdat_Parameter SPECNUMBER 
syn keyword upstreamdat_Parameter SPECTYPE 
syn keyword upstreamdat_Parameter SPECTYPE 
syn keyword upstreamdat_Parameter STARTTIME 
syn keyword upstreamdat_Parameter STORAGETYPE 
syn keyword upstreamdat_Parameter SUBDIRECTORIES 
syn keyword upstreamdat_Parameter SUBDIRECTORIES 
syn keyword upstreamdat_Parameter SWITCHTOTAPEMB 
syn keyword upstreamdat_Parameter TCPADDRESS 
syn keyword upstreamdat_Parameter TCPTIMEOUT 
syn keyword upstreamdat_Parameter TIMEOVERRIDE 
syn keyword upstreamdat_Parameter TRACE 
syn keyword upstreamdat_Parameter TRANSLATE 
syn keyword upstreamdat_Parameter ULTRACOMP 
syn keyword upstreamdat_Parameter ULTREG 
syn keyword upstreamdat_Parameter ULTUPD 
syn keyword upstreamdat_Parameter UNCMACHINEALIAS 
syn keyword upstreamdat_Parameter UNCMACHINEALIAS 
syn keyword upstreamdat_Parameter USEALEBRA 
syn keyword upstreamdat_Parameter USECONTROLFILE 
syn keyword upstreamdat_Parameter USEGID 
syn keyword upstreamdat_Parameter USERID 
syn keyword upstreamdat_Parameter USEUID 
syn keyword upstreamdat_Parameter USNOUIDGIDERRORS 
syn keyword upstreamdat_Parameter UTF8 
syn keyword upstreamdat_Parameter VAULTNUMBER 
syn keyword upstreamdat_Parameter VERSIONDATE 
syn keyword upstreamdat_Parameter WRITESPARSE 
syn keyword upstreamdat_Parameter XFERECORDSIZE 
syn keyword upstreamdat_Parameter XFERRECSEP 
syn keyword upstreamdat_Parameter XFERRECUSECR 

" File Specs:
syn match upstreamdat_Filespec /file spec\c \d\{1,3}.*/

" Comments:
syn match upstreamdat_Comment /^#.*/

" List Of Parameters:
syn region upstreamdat_Parms start="Current Parameters:" end="End Of Parameters" transparent fold

hi def link upstreamdat_Parameter Type
"hi def link upstreamdat_Filespec Underlined
hi def link upstreamdat_Comment Comment

let b:current_syntax = "upstreamdat"
