@echo off
rem To be used on MS-Windows for Visual C++ 2010 Express Edition
rem   aka Microsoft Visual Studio 10.0.
rem See INSTALLpc.txt for information.
@echo on

call "%VS100COMNTOOLS%vsvars32.bat"
